/* mksock -- create unix domain sockets.
   Copyright (C) 90, 91, 1995-2010 Free Software Foundation, Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* Written by Dmitry V. Levin <ldv@altlinux.org>
   based on mknod.c by David MacKenzie <djm@ai.mit.edu> et al.  */

#include <config.h>
#include <stdio.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "system.h"
#include "error.h"
#include "modechange.h"
#include "quote.h"

/* The official name of this program (e.g., no 'g' prefix).  */
#define PROGRAM_NAME "mksock"

#define AUTHORS proper_name ("Dmitry V. Levin")

static struct option const longopts[] =
{
  {"mode", required_argument, NULL, 'm'},
  {GETOPT_HELP_OPTION_DECL},
  {GETOPT_VERSION_OPTION_DECL},
  {NULL, 0, NULL, 0}
};

void
usage (int status)
{
  if (status != EXIT_SUCCESS)
    emit_try_help ();
  else
    {
      printf (_("Usage: %s [OPTION]... NAME...\n"), program_name);
      fputs (_("\
Create unix domain sockets with the given NAMEs.\n\
\n\
  -m, --mode=MODE    set file permission bits to MODE, not a=rw - umask\n\
"), stdout);
      fputs (HELP_OPTION_DESCRIPTION, stdout);
      fputs (VERSION_OPTION_DESCRIPTION, stdout);
      emit_ancillary_info ();
    }
  exit (status);
}

int
main (int argc, char **argv)
{
  mode_t newmode;
  const char *specified_mode = NULL;
  int exit_status = EXIT_SUCCESS;
  int optc;

  initialize_main (&argc, &argv);
  set_program_name (argv[0]);
  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE);

  atexit (close_stdout);

  while ((optc = getopt_long (argc, argv, "m:", longopts, NULL)) != -1)
    {
      switch (optc)
        {
        case 0:
          break;
        case 'm':
          specified_mode = optarg;
          break;
        case_GETOPT_HELP_CHAR;
        case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
        default:
          usage (EXIT_FAILURE);
        }
    }

  if (optind == argc)
    {
      error (0, 0, _("missing operand"));
      usage (EXIT_FAILURE);
    }

  newmode = (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
  if (specified_mode)
    {
      struct mode_change *change = mode_compile (specified_mode);
      if (!change)
        error (EXIT_FAILURE, 0, _("invalid mode"));
      newmode = mode_adjust (newmode, false, umask (0), change, NULL);
      free (change);
      if (newmode & ~S_IRWXUGO)
        error (EXIT_FAILURE, 0,
               _("mode must specify only file permission bits"));
    }

  for (; optind < argc; ++optind)
    {
      struct sockaddr_un sun;
      int fd;

      if (strlen (argv[optind]) >= sizeof (sun))
        {
          error (EXIT_SUCCESS, EINVAL, _("cannot bind socket `%s'"),
                 quote (argv[optind]));
          exit_status = EXIT_FAILURE;
          continue;
        }

      memset (&sun, 0, sizeof (sun));
      sun.sun_family = AF_UNIX;
      strcpy (sun.sun_path, argv[optind]);

      fd = socket (PF_UNIX, SOCK_STREAM, 0);
      if (fd < 0)
        {
          error (EXIT_SUCCESS, errno, _("cannot create socket `%s'"),
                 quote (argv[optind]));
          exit_status = EXIT_FAILURE;
          continue;
        }

      if (specified_mode && fchmod (fd, newmode))
        {
          error (EXIT_SUCCESS, errno, _("cannot set permissions of socket `%s'"),
                 quote (argv[optind]));
          close (fd);
          exit_status = EXIT_FAILURE;
          continue;
        }

      if (bind (fd, (struct sockaddr *) &sun, sizeof (sun)))
        {
          error (EXIT_SUCCESS, errno, _("cannot bind socket `%s'"),
                 quote (argv[optind]));
          exit_status = EXIT_FAILURE;
        }
      close (fd);
    }

  return exit_status;
}
